/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h> /* rand */
#include <math.h>
#include "offender.h"

#define LASER_INC 7

void pick_sprite_direction (struct unit *, double);

/* alien shoot.  randomly. */
static inline void a_shoot(struct unit *a)
{
	if (a->weapon->type & BULLET) {
		if((rand() % 200) < a->shootpct)
			make_unit(a->type->width / 2 + a->x,
				  a->type->height + a->y,
				  0, LASER_INC, LASER, 0, 0);
	} else {
		/* for extra alien weaponry */
		if((rand() % 200) < a->shootpct)
			make_unit(a->type->width / 2 + a->x,
				  a->type->height + a->y,
				  0, 0, a->weapon->type, 0, 0);
	}

}

/* args: unit structure, unit x val(it's x and old_x), strafing dist */
void strafe(struct unit *a, int *x, int dist)
{
	int offset, *ptr, inc_x;

	ptr = conf_a_from_type(conf, a->type->type);
	inc_x = ptr[INC_X];

	if(!a->form_strafe)	/* form_strafe is where it started strafing */
		a->form_strafe = *x;

	offset = *x - a->form_strafe;

	/* to handle independant setting of inc_x(rather than inc_x = -inc_x)*/
	if(abs(offset) >= dist) {		/* change direction */
		a->form_strafe = *x;		/* reset */
		if(offset < 0)
			*x += abs(inc_x);
		else
			*x -= abs(inc_x);
	} else if(offset < 0) {
		*x -= abs(inc_x);
	} else if (offset > 0) {
		*x += abs(inc_x);
	} else {
		if(a->x > a->form_strafe)
			*x += abs(inc_x);
		else if(a->x < a->form_strafe)
			*x -= abs(inc_x);
		else
			*x -= abs(inc_x);
	}
}

void strafe_incx(struct unit *a, int *x, int dist)
{
	a->inc_x = 0;		/* so it doesn't get 'inc'ed in do_units */
	strafe(a, x, dist);
}

/* this is the offender patented attacking engine that is so complicated
 * it cannot be understood without the comments. so, to preserve it's
 * integrity, we have encoded the comments with our patented secret algorithm

 * vs lbh'er ivrjvat jvgubhg xabjvat bhe frperg rapbqvat cebprff cyrnfr fgbc,
 * bgurejvfr gunax lbh, pbzr ntnva!
 * netf: gur havg, k vaperzrag, ubj znal k vaperzragf, l vaperzrag,
 *	 ubj zhpu gur havg fubhyq 'fjnl', urur, gbjneqf gur cynlre, curj :) */
int swerve_attack(struct unit *a, int inc_x, int times,
				int inc_y, int towards)
{
	g_assert(times>0);
	if(a->inc_y != 0) {	/* vg'f nggnpxvat */
		if(a->x < player->x)    /* jrvtugrq gbjneqf cynlre */
			a->x += towards/times;
		else if(a->x > player->x)
			a->x -= towards/times;
		if(a->form_swerve == times) {
			a->inc_x = -a->inc_x;	/* punatr qverpgvba */
			a->form_swerve = 0;	/* naq fgneg pbhag bire */
		}
		a->form_swerve++;
	} else { /* we're not swerving yet.. so randomly start */
		if(rand() % 250) return 0;	/* still not :( */
		a->shootpct += 3;		/* hubuu.. */
		a->form_swerve = 0;
		a->inc_y = inc_y;
		a->inc_x = inc_x;
	}
	return 1; 	/* jr'er fjreir_nggnpx()'vat */
}

int go_back(struct unit *a, int xspeed, int yspeed)
{
	if(OFF_BOTTOM(a->y)) {
		a->y = 0;
		a->form_swerve = -3;	/* we're not swerve_attacking anymore*/
	}

	if(a->form_swerve >= 0)
		return 0;

	a->form_swerve = -3;
	if(a->y > a->old_y) 	/* oops! overstepped */
		a->inc_y = -1;
	else if(a->y == a->old_y)
		a->inc_y = a->type->inc_y;

	if(a->x < a->old_x) {
		if(a->x < (a->old_x - 5))
			a->inc_x = xspeed;
		else
			a->x = a->old_x;
	} else if(a->x > a->old_x) {
		if(a->x > (a->old_x + 5))
			a->inc_x = -xspeed;
		else
			a->x = a->old_x;
	} else {
		a->inc_x = a->type->inc_x;
	}

	/* back in position clean up your mess, sloppy funtion... :) */
	if(a->x == a->old_x && a->y == a->old_y) {
		a->form_swerve = 0;
		a->shootpct = a->type->shootpct;
		return 0;
	}

	return 1;
}

void ai_normal(struct unit *a)
{
	int *ptr;
	if(!(a->type->type & BALIEN))	/* blue aliens shoot in their ai */
		a_shoot(a);

	ptr = conf_a_from_type(conf, a->type->type);

	if(!go_back(a, 3, 3) && !swerve_attack(a, ptr[ATK_X], ptr[TIMES],
				ptr[ATK_Y], ptr[TOWARDS]))
		strafe_incx(a, &a->x, ptr[STRF_DIST]);

	strafe(a, &a->old_x, ptr[STRF_DIST]);
	
	return;
}

void ai_blue_alien(struct unit *a)
{
	if((rand() % 100) < a->shootpct) {
		int third = a->type->width / 3;
		make_unit(third + a->x, a->type->height + a->y,
					0, LASER_INC, LASER, 0, 0);
		make_unit(third + third + a->x, a->type->height + a->y,
					0, LASER_INC, LASER, 0, 0);
	}
	ai_normal(a);
}

/* cloaks are hard to see aliens that slowly creep towards the player and dodge
 * his bullets ;) */
void ai_cloak(struct unit *a)
{
	struct unit *incoming;
	a_shoot(a);

	/* set inc_x and inc_y to go towards the player */
	if((a->inc_x < 0 && a->x < player->x) ||
	   (a->inc_x > 0 && a->x > player->x))
		a->inc_x = -a->inc_x; /* toggle */
	if((a->inc_y < 0 && a->y < player->y) ||
	   (a->inc_y > 0 && a->y > player->y))
		a->inc_y = -a->inc_y;

	/* dodge any incoming bullets :) within 20 pixels */
	incoming = type_in_range(BULLET, a->x, a->y, a->x+a->type->width,
						a->y+a->type->height+20);
	if(incoming)
		a->x += (rand() % 50) - 25;

	return;
}

/* blobs are entirely random except that they are more prone to go
 * to the top of the lower their health */
void ai_blob(struct unit *a)
{
	a_shoot(a);

	/* four calls, subtracting doesn't work :( */
	if ((rand () % 3) == 0) {
		/* set inc_x and inc_y to go towards the player */
		if (a->x < player->x)
			a->x += rand () % 8;
		else
			a->x -= rand () % 8;
		if (a->y < player->y)
			a->y += 2 * (rand () % 3 + 1);
		else
			a->y -= rand () % 4;
	} else {
		a->x += rand() % 8;
		a->x -= rand() % 8;
		a->y -= rand() % 4;
		a->y += rand() % (a->health % 10);
	}

	if (a->y > P1_START_Y - 20)
		a->y = P1_START_Y - 20;
	
	/* make sure it doesn't go off the screen */
	if(a->x > SCREEN_WIDTH - a->type->width)
		a->x = (SCREEN_WIDTH) - a->type->width;
	else if(a->x < 0)
		a->x = 0;
	if (a->y > SCREEN_HEIGHT - a->type->height)
		a->y = SCREEN_HEIGHT - a->type->height;
	else if(a->y < 0 )
		a->y = 0;
	return;
}

/* when hit a kami alien flies at the player, until then it acts like a normal
 * alien */
void ai_kami(struct unit *a)
{
	a_shoot(a);

	if(a->health < a->type->health) {
		a->inc_y = conf->kami[ATK_Y];
		if(a->x < player->x)    /* go towards the player */
			a->x += conf->kami[ATK_X];
		else if(a->x > player->x)
			a->x -= conf->kami[ATK_X];
		
		if( !(rand() % 7) ) /* ahh!! we're exploding!! .. heh.. */
			make_explo(a->x, a->y);
		
		if(a->y > player->y + player->type->height &&
				(rand() % RAND_MAX-1 ||
				 a->y > SCREEN_WIDTH - a->type->width)) {
			explo_destroy(a->x, a->y);
			a->health = 0;
		}
	} else {
		strafe_incx(a, &a->x, conf->kami[STRF_DIST]);
		strafe(a, &a->old_x, conf->kami[STRF_DIST]);
	}
}

void ai_laser(struct unit *a)
{
	gint type = PLAYER;
	struct unit *hit;

	if(a->type->type & BULLET)
		type = ALIENS;

	hit = type_in_range(type, a->x, a->y, a->x + a->type->width,
					a->y + a->type->height);
	if(hit != NULL) {
		if(hit->health <= 0)
			return;
		a->health = 0;

		if(type == ALIENS) {
	 		hit->health -= 4;
		} else if (type == PLAYER) {
			hit->health -= 1;
			/* The player died, get ready to rumble */
			if (hit->health == 0)
				switch_game_mode(STARTSCREEN);
			else
				switch_game_mode(GETREADY);
		} else {
			printf("huh?\n");
		}

		if (hit->health <= 0)	  //  * blow up the ship ;) *
			explo_destroy(hit->x, hit->y);
		else
			make_explo(hit->x, hit->y);
	}
}

void
ai_rocket(b)
	struct unit *b;
{
        gint type = ALIENS;
        struct unit *hit;

	  /* accelerate :) */
	if (b->inc_y > -19)
		b->inc_y--;

        hit = type_in_range(type, b->x, b->y, b->x + b->type->width,
                                        b->y + b->type->height);
        if(hit != NULL) {
                if(hit->health == 0)
                        return;
                b->health = 0;

		hit->health -= 8;

                if (hit->health <= 0)   //  * blow up the ship ;) *
                        explo_destroy(hit->x, hit->y);
                else
                        make_explo(hit->x, hit->y);
        }

}

/**********************************/
/* here comes the heat            */
/**********************************/

struct two_vec {
	double x;
	double y;
};

struct two_vec
v_difference (to, from)
	struct two_vec to, from;
{
	struct two_vec diff_vec;

	diff_vec.x = to.x - from.x;
	diff_vec.y = to.y - from.y;

	return diff_vec;
}

inline double
d_dot_v (v1, v2)
	struct two_vec v1, v2;
{
	return (v1.x * v2.x + v1.y * v2.y);
}

double
d_abs_v (vec)
	struct two_vec vec;
{
	double mag;
	mag = sqrt (d_dot_v (vec, vec));
	return mag;
}
/*
struct two_vec
rotate_v (vec, angle)
	struct two_vec vec;
	double angle;
{
	struct two_vec r_vec;

	printf ("rotating :  %f  : degrees \n", angle * 180 / M_PI);

	r_vec.x =  cos (angle) * vec.x - sin (angle) * vec.y;
	r_vec.y = -sin (angle) * vec.x + cos (angle) * vec.y;

	return r_vec;
}*/

struct two_vec
v_norm (vec)
	struct two_vec vec;
{
	double N, dist;
	struct two_vec norm;

	dist = sqrt (d_dot_v (vec, vec));

	N = 1.0 / dist;
	norm.x = N * vec.x;
	norm.y = N * vec.y;

	return norm;
}

struct two_vec
v_proj (vec, onto_vec)
	struct two_vec vec, onto_vec;
{
	struct two_vec proj_vec;
	double length;

	/* right direction */
	proj_vec = v_norm (onto_vec);
	length = d_abs_v (vec);

	/* right length */
	proj_vec.x *= length;
	proj_vec.y *= length;

	return proj_vec;
}

struct two_vec
close_target (heat_seeker)
	struct unit *heat_seeker;
{
	gint type = ALIENS;
	double dist = 0.0, prev_dist = -1.0, prev_dist1 = -1.0;
	double angle;
	struct two_vec hs_vec, target_vec, diff_vec, inc_vec, accel_vec;
	struct two_vec r_vec;
	GList *node = units, *next;
	struct unit *enemy = NULL, *closest = NULL;

	hs_vec.x = (double) heat_seeker->x;
	hs_vec.y = (double) heat_seeker->y;

	//printf("marker 1\n");
	for (; node; node = next) {
		next = node->next;
		enemy = (struct unit *) node->data;

		if (!(type & enemy->type->type))
			continue;
		if (prev_dist == -1.0)
			closest = (struct unit*) node->data;

		target_vec.x = (double) enemy->x;
		target_vec.y = (double) enemy->y;

		diff_vec = v_difference (target_vec, hs_vec);
		dist = d_abs_v (diff_vec);
		prev_dist = (prev_dist == -1.0) ? dist : prev_dist;
		prev_dist1 = (prev_dist1 == -1.0) ? dist : prev_dist1;

//		printf ("prev1: %d \t prev: %d \t dist: %d\n", prev_dist1,
//				prev_dist, dist);
//		printf ("x: %d y: %d to x: %f y: %f \n", enemy->x, enemy->y,
//				hs_vec.x, hs_vec.y);
		/* current or previous dist greater ? */
		if (prev_dist1 != MIN(dist, prev_dist1)) {
//			printf (" small \n");
			prev_dist1 = dist;
			closest = (struct unit *) node->data;
//			printf ("closest: %d\n", closest->x);
			continue;
		}

		if (prev_dist1 < prev_dist) {
			prev_dist = prev_dist1;
//			printf ("----here ? closest: %d\n", closest->x);
			closest = (struct unit *) node->prev->data;
		}
	}
//	printf("min unit?  x: %d y: %d \n", closest->x, closest->y);

	if (!closest) {
		target_vec.x = 0.0;
		target_vec.y = 0.0;
		return target_vec;
	}

	target_vec.x = (double) closest->x;
	target_vec.y = (double) closest->y;

	return target_vec;
}

/* accelerate towards the enemy */
void
turn_heat_to (heat_seeker, target_vec)
	struct unit *heat_seeker;
	struct two_vec target_vec;
{
	double angle;
	double N, dot;
	struct two_vec norm_vec;
	struct two_vec horiz_vec = {1.0, 0.0};
	struct two_vec inc_vec, r_vec, accel_vec, diff_vec, hs_vec;

	if (target_vec.x == 0.0 && target_vec.y == 0.0) {
		heat_seeker->inc_y -= 1;
		angle = M_PI / 2.0;
		pick_sprite_direction (heat_seeker, angle);
		return;
	}

	hs_vec.x = (double) heat_seeker->x;
	hs_vec.y = (double) heat_seeker->y;

	/* vector from heat seeker to target */
	diff_vec = v_difference (target_vec, hs_vec);

	/* interested in the increments to the velocity */
	if (heat_seeker->inc_x == 0) {
		inc_vec.x = (double) heat_seeker->inc_x - .01;
		inc_vec.y = (double) heat_seeker->inc_y + .01;
	} else {
		inc_vec.x = (double) heat_seeker->inc_x;
		inc_vec.y = (double) heat_seeker->inc_y;
	}


	/* project the increment to the right direction. *
	 * accel is the change in increment		*/
	r_vec = v_proj (inc_vec, diff_vec);
	accel_vec = v_difference (r_vec, inc_vec);

	diff_vec = v_norm (diff_vec);
	accel_vec = v_norm (accel_vec);

	/* add diff_vec on so we rotate and accel towards target */
	accel_vec.x += diff_vec.x;
	accel_vec.y += diff_vec.y;

	/* don't accel too much */
	norm_vec = v_norm (accel_vec);

	heat_seeker->inc_x += (int) round (norm_vec.x);
	heat_seeker->inc_y += (int) round (norm_vec.y);
/*
	printf ("---------------------------------------------\n");
	printf ("inc vec:  %f\t %f\n", inc_vec.x, inc_vec.y);
	printf ("diff vec:  %f\t %f\n", diff_vec.x, diff_vec.y);
	printf ("r vec  :  %f\t %f\n", r_vec.x, r_vec.y);
	printf ("hs vec :  %f\t %f\n", hs_vec.x, hs_vec.y);
	printf ("target :  %f\t %f\n", target_vec.x, target_vec.y);
	printf ("----------------------------------------------\n");
	printf ("^^^^^ norm vec x: %f y: %f\n", norm_vec.x,  norm_vec.y);

*/
	/* angle between heat to target(diff) and the horizontal *
	*  always positive (problem) angle between the two vecs. */
	angle = acos ( (d_dot_v (horiz_vec, inc_vec)) 
			/ (d_abs_v (horiz_vec) * d_abs_v (inc_vec)) );

	/* In order to point the sprite in the right direction     */
	/* need to know what hand on the clock the target is.  but *
	 * we use a 1 -> -1 (pi to -pi) interval for the angle, so *
	 * if target is below heat then rotate cw, otherwise vice  *
	 * versa.  this corresponds to setting angle + or -.       */
	/* NOTE: rotating image from x dir */
	{
		/* slightly counter-intuitive coord. notation */
		struct two_vec unity_vec = {0.0, -1.0}, yproj_vec;

		yproj_vec = v_proj (diff_vec, unity_vec);
		yproj_vec = v_norm (yproj_vec);

		/* if target below x of heat, rotate minus */
		if (yproj_vec.y != unity_vec.y)
			angle = -angle;
	}

	/* now that we have the right angle for image, pick it */
	pick_sprite_direction (heat_seeker, angle);
}

void
pick_sprite_direction (heat_seeker, angle)
	struct unit *heat_seeker;
	double angle;
{
	double eth = 1.0/8.0, qtr = 1.0/4.0;

	angle /= M_PI;  /* set to units of pi */

//	printf ("****  here ****   angle : %f %f \n", angle, 1/8.0);
	if ((angle < eth ) && (angle >= -eth))
		heat_seeker->seq = 1;
	else if ( (angle >=  eth) && (angle < (eth + qtr)) )
		heat_seeker->seq = 3;
	else if ( (angle >= (eth + qtr)) && (angle < (eth + 2.0 * qtr)) )
		heat_seeker->seq = 5;
	else if ( (angle >= (eth + 2.0 * qtr)) && (angle < (eth + 3.0 * qtr)) )
		heat_seeker->seq = 7;
	else if ( (angle >=  (eth + 3.0 * qtr) && (angle <= 1))
			|| (angle < (-eth - 3.0 * qtr)) && (angle >= -1) )
		heat_seeker->seq = 9;
	else if ( (angle <= (-eth -qtr)) && (angle > (-eth - 2.0 * qtr)) )
		heat_seeker->seq = 11;
	else if ( (angle <=  (-eth - 2.0 * qtr))
			&& (angle > (-eth  - 3.0 * qtr)) )
		heat_seeker->seq = 13;
	else if ( (angle <= (-eth - 3.0 * qtr) && (angle >= -1))
			|| (angle > (eth + 3.0 * qtr)) && (angle <= 1) )
		heat_seeker->seq = 15;
	else 
		printf ("Error: pick_sprite_direction()");

//	printf ("seq # %d \n", heat_seeker->seq);
}


void
ai_heats (b)
	struct unit *b;
{
	gint type = ALIENS;
	struct unit *hit;
	struct two_vec target;

	/* find closest target and accelerate towards it */
	target = close_target (b);
	turn_heat_to (b, target);

	b->seq--;

	hit = type_in_range(type, b->x, b->y, b->x + b->type->width,
					b->y + b->type->height);
	if(hit != NULL) {
		if(hit->health == 0)
			return;
		b->health = 0;
		if (--hit->health == 0)   //  * blow up the ship ;) *
			explo_destroy(hit->x, hit->y);
		else
			make_explo(hit->x, hit->y);
	}

}



/* for explosions */
void ai_dec_health(struct unit *a)
{
	a->health--;
}

gint tank_shoot(gpointer data)
{
	struct unit *t = (struct unit *) data;
	make_unit(t->x+t->type->width/2, t->y+t->type->height,
					0, 10, LASER, 0, 0);
        return 1;
}

/* weapon display item 'ai' */
void
ai_dsp (d)
	struct unit *d;
{
	/* health display */
	if (d->type->type == NUM) {
		d->seq = player->health - 2;
		return;
	}

	if (d->weapon == NULL)
		return;

	/* stay at first sprite (Note: seq is incremented in
	* the function sequence_sprite() that's why '-1'    */
	d->seq = -1;

	/* highlight weapon sprite if selected / current weapon */
	if (player->weapon->type == d->weapon->type)
		++d->seq;
}

void ai_background (struct unit *bg)
{
	gint width;
	gint height;
	gint top;
	
	if(game_mode != PLAYING) {
		background->y = 0;  /* only scrolls while playing */
		return;
	}

	sscanf(bg->type->sprites[bg->seq].xpm_data[0], "%d %d",
			&width, &height);

	/* if we're done with this run of the background.  recommence */
	if(bg->y >= 0) {
		/* show at least the first screen height */
		bg->y = SCREEN_HEIGHT - height + bg->inc_y;
	}

}

