/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <glib.h>
#include <stdio.h>

#include "offender.h"
#include "ui.h"

extern GtkWidget *window;

/* Backing Pixmap for Drawing */
static GdkPixmap *pixmap = NULL;

void load_pixmap(GtkWidget *widget, struct a_type *type)
{
	int i = 0;

	while (type->sprites[i].xpm_data) {
		type->sprites[i].pixmap = gdk_pixmap_create_from_xpm_d (
					widget->window, &type->sprites[i].mask,
					NULL, type->sprites[i].xpm_data );
		i++;
	}

	sscanf(type->sprites[0].xpm_data[0], "%d %d",
					&type->width, &type->height);
	type->nr_seqs = i;
}

void draw_sprite(GtkWidget *area, struct unit *a)
{
	gint height;
	gint width;
	
	GdkGC *gc = area->style->fg_gc[GTK_STATE_NORMAL];

	gdk_gc_set_clip_mask (gc, a->type->sprites[a->seq].mask);
	gdk_gc_set_clip_origin (gc, a->x, a->y);

	/* well, you ask why..  why do we need to do this again when we've 
	 * clearly done it already and it's in the type structure variables 
	 * of the same name?  Well cappin, is because of the background type 
	 * which is made up of various sprites of different heights.  
	 * thusly and thereforth, this is of necessity. */	
	sscanf(a->type->sprites[a->seq].xpm_data[0], "%d %d",
			&width, &height);

	/* copy pixmap to the window, properly clipped */
	gdk_draw_pixmap(pixmap, gc, a->type->sprites[a->seq].pixmap, 0, 0,
			a->x, a->y, width, height);

	gdk_gc_set_clip_mask (gc, NULL);

}

void sequence_sprite(GtkWidget *area, struct unit *a)
{
	if (++a->seq >= a->type->nr_seqs)
		a->seq = 0;

	draw_sprite(area, a);
}

gint repaint(gpointer data)
{
	GtkWidget *area = (GtkWidget *) data;
	GdkRectangle update_rect;

	if(game_mode != PAUSED) {
		if(conf->usekeyboard == TRUE)
			handle_them_keys(area->allocation.width,
					  area->allocation.height);
		else
			handle_that_mouse();
	}

	/* draw the background */
	gdk_draw_rectangle (pixmap, area->style->black_gc, TRUE, 0, 0,
			area->allocation.width, area->allocation.height);


	if(game_mode == GETREADY) {
		if(--get_ready_timeout <= 0)
			switch_game_mode(PLAYING);
	}

	if(game_mode != PLAYING) {
		draw_sprite(drawing_area, background);
	} else {
		/* the all important function */
		do_units();
	}

	/* The whole screen needs to be updated */
	update_rect.x = 0;
	update_rect.y = 0;
	update_rect.width = area->allocation.width;
	update_rect.height = area->allocation.height;
	
	gtk_widget_draw (area, &update_rect);

	return 1;
}

gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
	gdk_draw_pixmap (widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap, 
		event->area.x, event->area.y, event->area.x, event->area.y,
		event->area.width, event->area.height);

	return FALSE;
}

/* called whenever the window is resized or moved. */
gint config_event(GtkWidget *widget, GdkEventConfigure *event)
{
	/* free background if we created it */
	if (pixmap)
		gdk_pixmap_unref (pixmap);

	pixmap = gdk_pixmap_new (widget->window, widget->allocation.width,
				 widget->allocation.height, -1);

	return TRUE;
} 
