/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "offender.h"
#include <gdk/gdkkeysyms.h>
#include <string.h>
#include <stdio.h>

#define assign_a(alien, x, y, ax, t, ay, to, dis, he, sh) {	\
	alien[INC_X] = x;		alien[INC_Y] = y;	\
	alien[ATK_X] = ax;		alien[TIMES] = t;	\
	alien[ATK_Y] = ay;		alien[TOWARDS] = to;	\
	alien[STRF_DIST] = dis;		alien[HEALTH] = he;	\
	alien[SHOOTPCT] = sh;					\
}

GtkWidget *conf_window;

struct config *conf = NULL;
struct config *temp_conf = NULL;

// for buttons
static GdkColor black = {0, 0x0000, 0x0000, 0x0000};
static GdkColor blue  = {0, 0x0000, 0x0000, 0xffff};
static GdkColor green = {0, 0x0000, 0xffff, 0x0000};
static GdkColor red   = {0, 0xffff, 0x0000, 0x0000};

void make_game_page(GtkWidget *);
void make_control_page(GtkWidget *);
GtkStyle *set_button_color(GdkColor fg, GdkColor text, GdkColor bg);
void change_diff(struct config *, gint);

void init_config()
{
	temp_conf = (struct config *)g_malloc (sizeof(struct config));
	conf = (struct config *)g_malloc (sizeof(struct config));
	conf->diff_level = 1;
	conf->usekeyboard = TRUE;
	conf->keyleft = GDK_Left;
	conf->keyright = GDK_Right;
	conf->keyup = GDK_Up;
	conf->keydown = GDK_Down;
	conf->keyshoot = GDK_space;
	conf->keychg_weapon = GDK_c;
	conf->keystart = GDK_s;
	conf->start_level = 1;
	conf->start_lives = 3;
	conf->max_bullets = 5;
	
	change_diff(conf, EASY);

	memcpy(temp_conf, conf, sizeof(struct config));
}

GtkWidget *config_add_page(GtkWidget *notebook, gint tf, char *arg_label)
{
	GtkWidget *label = gtk_label_new (arg_label);
	GtkWidget *vbox = gtk_vbox_new(tf, 0);

        gtk_notebook_append_page(GTK_NOTEBOOK (notebook), vbox, label);
	return vbox;
}

void save_clicked() {
	memcpy(conf, temp_conf, sizeof(struct config));
	gtk_widget_destroy(conf_window);
}

void quit_clicked() {
	memcpy(temp_conf, conf, sizeof(struct config)); // reset temp_conf
	gtk_widget_destroy(conf_window);
}

/*
 * I probably should have named this function:
 * gtk_make_the_config_window_tabs_notebook_and_pages_so_the_user_can_
 * configure_the_game_and_have_more_fun() but i decided not to. :) 
 */
void config_make()
{
	GtkWidget *notebook = gtk_notebook_new();
	GtkWidget *game_box, *ctrl_box, *advn_box;
	GtkWidget *save, *quit;
	
	conf_window = gtk_dialog_new();

	gtk_window_set_title (GTK_WINDOW (conf_window), "Configure");
	gtk_container_border_width (GTK_CONTAINER (conf_window), 0);
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_RIGHT);

	game_box = config_add_page(notebook, TRUE, "Game");
	ctrl_box = config_add_page(notebook, FALSE, "Control");
	
	make_game_page(game_box);
	make_control_page(ctrl_box);
	
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (conf_window)->vbox),
				notebook, TRUE, TRUE, 0);

	// save button
	save = gtk_button_new_with_label("Save");
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (conf_window)->action_area), 
				save, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (save), "clicked", GTK_SIGNAL_FUNC
				(save_clicked), NULL);
	// quit without saving
	quit = gtk_button_new_with_label("Quit without Saving");
	gtk_widget_set_style(quit, set_button_color(black, black, red)); 
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (conf_window)->action_area), 
				quit, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (quit), "clicked", GTK_SIGNAL_FUNC
				(quit_clicked), NULL);
	// catch other means of closing
	gtk_signal_connect (GTK_OBJECT (conf_window), "delete_event",
				GTK_SIGNAL_FUNC(quit_clicked), NULL);

	gtk_widget_show_all(conf_window);
} 


static void config_keychange(GtkWidget *widget, GdkEventKey *event,
				gpointer data)
{
	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
	*(gint *)data = event->keyval;
}

GtkWidget *make_entry_box(char *txt)
{
	GtkWidget *entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(entry), txt);
	gtk_widget_show(entry);
	return entry;
}

static void use_keyb(GtkWidget *widget, gpointer data) {
        temp_conf->usekeyboard = (gint) data;
}

GtkWidget *make_radio_button(GSList *radio_group, char *label,
				GtkStyle *style, GtkWidget *packto)
{
	GtkWidget *radio = gtk_radio_button_new_with_label(radio_group, label);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio), TRUE);
	if(style)
	        gtk_widget_set_style(radio, style);
        gtk_box_pack_start (GTK_BOX (packto), radio, TRUE, TRUE, 0);
        gtk_widget_show(radio);
	return radio;
}

void make_control_page(GtkWidget *page)
{
	GtkWidget *hbox_all = gtk_hbox_new(FALSE, FALSE);
	GtkWidget *frame_keyb = gtk_frame_new("Keyboard Controls");
	GtkWidget *table = gtk_table_new(2, 3, TRUE);
	GtkWidget *hbox_keyblt = gtk_hbox_new(FALSE, FALSE);
	GtkWidget *hbox_keybrt = gtk_hbox_new(FALSE, FALSE);
	GtkWidget *hbox_keybup = gtk_hbox_new(FALSE, FALSE);
	GtkWidget *hbox_keybdn = gtk_hbox_new(FALSE, FALSE);
	GtkWidget *hbox_keybsh = gtk_hbox_new(FALSE, FALSE);
	GtkWidget *keyblt = make_entry_box(gdk_keyval_name(conf->keyleft));
	GtkWidget *keybrt = make_entry_box(gdk_keyval_name(conf->keyright));
	GtkWidget *keybup = make_entry_box(gdk_keyval_name(conf->keyup));
	GtkWidget *keybdn = make_entry_box(gdk_keyval_name(conf->keydown));
	GtkWidget *keybsh = make_entry_box(gdk_keyval_name(conf->keyshoot));
	GtkWidget *label_keyblt = gtk_label_new("Left:");
	GtkWidget *label_keybrt = gtk_label_new("Right:");
	GtkWidget *label_keybup = gtk_label_new("Up:");
	GtkWidget *label_keybdn = gtk_label_new("Down:");
	GtkWidget *label_keybsh = gtk_label_new("Shoot:");

	GtkWidget *usemouse_radio, *usekeyb_radio;
	GtkWidget *frame_usefu = gtk_frame_new("Input Device");
	GtkWidget *vbox_usefu = gtk_vbox_new(FALSE, FALSE);
	GSList *radio_group = NULL;

	gtk_box_pack_start(GTK_BOX(page), hbox_all, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame_usefu), vbox_usefu);
	gtk_box_pack_start(GTK_BOX(hbox_all), frame_usefu, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame_keyb), table);
	gtk_box_pack_start(GTK_BOX(hbox_all), frame_keyb, FALSE, FALSE, 0);
	
	// use keyboard/use mouse
	usekeyb_radio = gtk_radio_button_new_with_label (NULL, "Use Keyboard");
	usemouse_radio = gtk_radio_button_new_with_label_from_widget(
			GTK_RADIO_BUTTON(usekeyb_radio), "Use Mouse");

	if(conf->usekeyboard == TRUE)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(usekeyb_radio),
					     TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(usemouse_radio),
					     TRUE);

	gtk_signal_connect(GTK_OBJECT(usekeyb_radio), "toggled",
                        GTK_SIGNAL_FUNC(use_keyb), (gpointer) TRUE);
	gtk_signal_connect(GTK_OBJECT(usemouse_radio), "toggled",
			GTK_SIGNAL_FUNC(use_keyb), (gpointer) FALSE);

	gtk_box_pack_start(GTK_BOX(vbox_usefu), usekeyb_radio, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox_usefu), usemouse_radio,FALSE, FALSE, 0);

	// keyboard controls
	gtk_box_pack_start_defaults(GTK_BOX(hbox_keyblt), label_keyblt);
	gtk_box_pack_start_defaults(GTK_BOX(hbox_keyblt), keyblt);
	gtk_table_attach_defaults(GTK_TABLE(table), hbox_keyblt, 0, 1, 1, 2);
	gtk_widget_show(label_keyblt);
        gtk_widget_show(hbox_keyblt);
	gtk_signal_connect(GTK_OBJECT(keyblt), "key_press_event",
				GTK_SIGNAL_FUNC(config_keychange),
				&temp_conf->keyleft);

        gtk_box_pack_start_defaults(GTK_BOX(hbox_keybrt), label_keybrt);
        gtk_box_pack_start_defaults(GTK_BOX(hbox_keybrt), keybrt);
        gtk_table_attach_defaults(GTK_TABLE(table), hbox_keybrt, 2, 3, 1, 2);
	gtk_widget_show(label_keybrt);
	gtk_widget_show(hbox_keybrt);
	gtk_signal_connect(GTK_OBJECT(keybrt), "key_press_event",
				GTK_SIGNAL_FUNC(config_keychange),
				&temp_conf->keyright);

        gtk_box_pack_start_defaults(GTK_BOX(hbox_keybup), label_keybup);
        gtk_box_pack_start_defaults(GTK_BOX(hbox_keybup), keybup);
        gtk_table_attach_defaults(GTK_TABLE(table), hbox_keybup, 1, 2, 0, 1);
	gtk_widget_show(label_keybup);
	gtk_widget_show(hbox_keybup);
	gtk_signal_connect(GTK_OBJECT(keybup), "key_press_event",
				GTK_SIGNAL_FUNC(config_keychange),
				&temp_conf->keyup);

        gtk_box_pack_start_defaults(GTK_BOX(hbox_keybdn), label_keybdn);
        gtk_box_pack_start_defaults(GTK_BOX(hbox_keybdn), keybdn);
        gtk_table_attach_defaults(GTK_TABLE(table), hbox_keybdn, 1, 2, 2, 3);
	gtk_widget_show(label_keybdn);
	gtk_widget_show(hbox_keybdn);
	gtk_signal_connect(GTK_OBJECT(keybdn), "key_press_event",
				GTK_SIGNAL_FUNC(config_keychange),
				&temp_conf->keydown);

	gtk_box_pack_start_defaults(GTK_BOX(hbox_keybsh), label_keybsh);
	gtk_box_pack_start_defaults(GTK_BOX(hbox_keybsh), keybsh);
	gtk_table_attach_defaults(GTK_TABLE(table), hbox_keybsh, 0, 1, 4, 5);
	gtk_widget_show(label_keybsh);
	gtk_widget_show(hbox_keybsh);
	gtk_signal_connect(GTK_OBJECT(keybsh), "key_press_event",
				GTK_SIGNAL_FUNC(config_keychange),
				&temp_conf->keyshoot);
}

void change_diff(struct config *c, gint diff_level)
{
	if(c->diff_level == EASY) {
//: inc_x, inc_y, atk_x, times, atk_y, towards, strf_dist, health, shootpct
		assign_a(c->galien, -4, 0, 10, 7, 5, 17, 70, 4, 1);
		assign_a(c->balien, -4, 0, 10, 7, 5, 17, 70, 4, 1);
		assign_a(c->ralien, -4, 0, 10, 7, 5, 17, 70, 4, 1);
		assign_a(c->blob,    0, 0,  0, 0, 0,  0,  0, 8, 2);
		assign_a(c->cloak,   1, 1,  0, 0, 0,  0,  0, 4, 1);
		assign_a(c->kami,   -4, 0,  4, 0, 5,  0, 70, 6, 1);
		assign_a(c->tank,    0, 0,  0, 0, 0,  0,  0, 4, 0);
	} else if(c->diff_level == NORMAL) {
		assign_a(c->galien, -6, 0, 12, 7, 6, 19, 70, 4, 2);
		assign_a(c->balien, -6, 0, 12, 7, 6, 19, 70, 4, 1);
		assign_a(c->ralien, -6, 0, 12, 7, 6, 19, 70, 4, 3);
		assign_a(c->blob,    0, 0,  0, 0, 0,  0,  0, 8, 3);
		assign_a(c->cloak,   2, 2,  0, 0, 0,  0,  0, 4, 2);
		assign_a(c->kami,   -6, 0,  5, 0, 7,  0, 70, 3, 2);
		assign_a(c->tank,    0, 0,  0, 0, 0,  0,  0, 4, 0);
	} else if(c->diff_level == HARD) {
		assign_a(c->galien, -8, 0, 14, 8, 7, 23, 70, 8, 3);
		assign_a(c->balien, -8, 0, 14, 8, 7, 23, 70, 8, 2);
		assign_a(c->ralien, -8, 0, 14, 8, 7, 23, 70, 8, 4);
		assign_a(c->blob,    0, 0,  0, 0, 0,  0,  0, 12, 4);
		assign_a(c->cloak,   3, 3,  0, 0, 0,  0,  0, 8, 3);
		assign_a(c->kami,   -8, 0,  6, 0, 8,  0, 70, 12, 3);
		assign_a(c->tank,    0, 0,  0, 0, 0,  0,  0, 4, 0);
	}
}


void diff_radio_toggled (GtkWidget *widget, gpointer data)
{
	temp_conf->diff_level = (gint) data;
	change_diff(temp_conf, temp_conf->diff_level);
}

static void change_start_lives(GtkWidget *widget, gpointer data) {
	temp_conf->start_lives = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(data));
	printf("%d", temp_conf->start_lives);
}

static void change_start_level(GtkWidget *widget, gpointer data) {
	temp_conf->start_level = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(data));
}

static void change_max_bullets(GtkWidget *widget, gpointer data) {
	temp_conf->max_bullets = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(data));
}

GtkWidget *sbutton_with_label(char *name, gint start, gint from, gint to,
			void (*func)(GtkWidget *widget, gpointer data))
{
	GtkWidget *hbox = gtk_hbox_new(FALSE, FALSE);
	GtkWidget *label = gtk_label_new(name);
	GtkObject *adjustment = gtk_adjustment_new(
		(gfloat) start,
		(gfloat) from,
		(gfloat) to,
		1.0, 5.0, 1.0);
	GtkWidget *sbutton = gtk_spin_button_new(
				GTK_ADJUSTMENT(adjustment), 1.0, 0);
	
	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(sbutton), FALSE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(sbutton), TRUE);
	gtk_signal_connect(GTK_OBJECT(adjustment),
			"value_changed", GTK_SIGNAL_FUNC(func),
			sbutton);
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), sbutton, FALSE, FALSE,0);

	return hbox;
}

void make_game_page (GtkWidget *page)
{
	GtkWidget *easy_radio, *norm_radio, *hard_radio;
	GtkWidget *diff_vbox = gtk_vbox_new(FALSE, FALSE);
	GtkWidget *diff_frame = gtk_frame_new("Difficulty");
	GSList *radio_group = NULL;
        GtkStyle *red_style = set_button_color(black, black, red);
	GtkStyle *blue_style = set_button_color(black, black, blue);
	GtkStyle *green_style = set_button_color(black, black, green);

	GtkWidget *cheats_frame = gtk_frame_new("Cheats");
	GtkWidget *cheats_vbox = gtk_vbox_new(FALSE, FALSE);

	GtkWidget *info_frame = gtk_frame_new("Note");
	GtkWidget *info_vbox = gtk_vbox_new(FALSE, FALSE); // necessary
	GtkWidget *info_label = gtk_label_new("Changes to the Difficulty and Cheats\nsections will on apply after a new game is started.");
	
	
	GtkWidget *level_hbox = sbutton_with_label("Starting Level: ",
			conf->start_level, 1, NUM_LEVELS,
			change_start_level);

	GtkWidget *lives_hbox = sbutton_with_label("Starting Lives: ",
			conf->start_lives, 1, MAX_LIVES,
			change_start_lives);

	GtkWidget *maxbullets_hbox = sbutton_with_label(
			"Maximum Bullets on Screen: ",
			conf->max_bullets, 1, 100,
			change_max_bullets);

	GtkWidget *all_hbox = gtk_hbox_new(FALSE, FALSE);
	
	easy_radio = make_radio_button (NULL, "Easy", blue_style, diff_vbox);
	radio_group = gtk_radio_button_group (GTK_RADIO_BUTTON(easy_radio));
	norm_radio = make_radio_button (radio_group, "Normal", green_style,
			diff_vbox);
	radio_group = gtk_radio_button_group (GTK_RADIO_BUTTON(norm_radio));
	hard_radio = make_radio_button (radio_group, "Hard", red_style,
			diff_vbox);

	gtk_signal_connect (GTK_OBJECT (easy_radio), "clicked",
			GTK_SIGNAL_FUNC (diff_radio_toggled), (gpointer)EASY);
	gtk_signal_connect (GTK_OBJECT (norm_radio), "clicked",
			GTK_SIGNAL_FUNC (diff_radio_toggled), (gpointer)NORMAL);
        gtk_signal_connect (GTK_OBJECT (hard_radio), "clicked",
			GTK_SIGNAL_FUNC (diff_radio_toggled), (gpointer)HARD);
	
	// set the button to whatever the current difficulty level is
	if (conf->diff_level == EASY)
		gtk_signal_emit_by_name (GTK_OBJECT (easy_radio), "clicked");
	else if (conf->diff_level == NORMAL)
                gtk_signal_emit_by_name (GTK_OBJECT (norm_radio), "clicked");
	else if (conf->diff_level == HARD)
                gtk_signal_emit_by_name (GTK_OBJECT (hard_radio), "clicked");

	// pack everything on
	gtk_container_add(GTK_CONTAINER(diff_frame), diff_vbox);
	gtk_box_pack_start (GTK_BOX(all_hbox), diff_frame, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(cheats_frame), cheats_vbox);
	gtk_box_pack_start (GTK_BOX(all_hbox), cheats_frame, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(info_frame), info_vbox);
	gtk_box_pack_start (GTK_BOX(info_vbox), info_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(all_hbox), info_frame, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(cheats_vbox), level_hbox, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(cheats_vbox), lives_hbox, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(cheats_vbox), maxbullets_hbox, FALSE,
			FALSE, 0);
	gtk_box_pack_start (GTK_BOX(page), all_hbox, FALSE, FALSE, 0);
	
	gtk_widget_show(diff_frame);
	gtk_widget_show(cheats_frame);
}

int *conf_a_from_type(struct config *cf, int type)
{
	int *ptr;
	if(type & GALIEN)
		ptr = cf->galien;
	else if(type & BALIEN)
		ptr = cf->balien;
	else if(type & RALIEN)
		ptr = cf->ralien;
	else if(type & BLOB)
		ptr = cf->blob;
	else if(type & CLOAK)
		ptr = cf->cloak;
	else if(type & KAMI)
		ptr = cf->kami;
	else if(type & TANK)
		ptr = cf->tank;
	return ptr;
}

void update_types()
{
	change_type(GALIEN, conf->galien[INC_X], conf->galien[INC_Y], 
			conf->galien[HEALTH], conf->galien[SHOOTPCT]);
	change_type(BALIEN, conf->balien[INC_X], conf->balien[INC_Y], 
			conf->balien[HEALTH], conf->balien[SHOOTPCT]);
	change_type(RALIEN, conf->ralien[INC_X], conf->ralien[INC_Y], 
			conf->ralien[HEALTH], conf->ralien[SHOOTPCT]);
	change_type(BLOB, conf->blob[INC_X], conf->blob[INC_Y], 
			conf->blob[HEALTH], conf->blob[SHOOTPCT]);
	change_type(CLOAK, conf->cloak[INC_X], conf->cloak[INC_Y], 
			conf->cloak[HEALTH], conf->cloak[SHOOTPCT]);
	change_type(KAMI, conf->kami[INC_X], conf->kami[INC_Y], 
			conf->kami[HEALTH], conf->kami[SHOOTPCT]);
	change_type(TANK, conf->tank[INC_X], conf->tank[INC_Y], 
			conf->tank[HEALTH], conf->tank[SHOOTPCT]);

	return;
}

GtkStyle *set_button_color(GdkColor fg, GdkColor text, GdkColor bg)
{
	GtkStyle *def_style, *new_style;
	gint tmp;
 
	/* Get the default style and make a copy. */
	def_style = gtk_widget_get_default_style();
	new_style = gtk_style_copy(def_style);

	for (tmp = 0; tmp < 5; tmp++) {
		new_style->fg[tmp]   = fg;
		new_style->text[tmp] = text;
		new_style->bg[tmp]   = bg;
	}
	return (new_style);
}
