/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "offender.h"
#include "ui.h"
#include "pixmaps/cursor.h"

#define UP		1
#define DOWN		2
#define LEFT		4
#define RIGHT		8
#define SHOOT		16
#define CHG_WEAPON 	32
#define START		64

int keys = 0;

void masked_bitmap_from_xpm (GdkBitmap **bitmap, GdkBitmap **mask, gchar **xpm);
void plshoot();

void track_keys (GtkWidget *widget, GdkEventKey *eventkey)
{
	if(eventkey->keyval == conf->keyleft)
		keys |= LEFT;
	else if(eventkey->keyval == conf->keyright)
		keys |= RIGHT;
	else if(eventkey->keyval == conf->keyup)
		{/*keys |= UP;*/}
	else if(eventkey->keyval == conf->keydown)
		{/*keys |= DOWN;*/}
	else if(eventkey->keyval == conf->keyshoot)
		keys |= SHOOT;
	else if(eventkey->keyval == conf->keychg_weapon)
		keys |= CHG_WEAPON;
	else if(eventkey->keyval == conf->keystart)
		keys |= START;
}


void release_keys (GtkWidget *widget, GdkEventKey *eventkey)
{	
        if(eventkey->keyval == conf->keyleft)
                keys ^= LEFT;
        else if(eventkey->keyval == conf->keyright)
                keys ^= RIGHT;
        else if(eventkey->keyval == conf->keyup)
	{/*keys ^= UP;*/}
        else if(eventkey->keyval == conf->keydown)
	{/*keys ^= DOWN;*/}
}

void clear_keys()
{
	keys = 0;
}

void handle_them_keys(gint width, gint height)
{
	// enforcing boundries isn't fun ;)
	if (keys & UP) {
		if(player->y > player->inc_y) 
			player->y -= player->inc_y;
		else 
			player->y = 0;
	}
	if (keys & LEFT) {
		if(player->x > player->inc_x)
			player->x -= player->inc_x;
		else
			player->x = 0;
	}
	if (keys & DOWN) {
		if (player->y + player->type->height < height - player->inc_y)
			player->y += player->inc_y;
		else
			player->y = height - player->type->height;
	}
	if (keys & RIGHT) {
		if ((player->x + player->type->width) < width - player->inc_x)
			player->x += player->inc_x;
		else
			player->x = width - player->type->width;
	}
	if (keys & SHOOT) {
		plshoot();
		keys ^= SHOOT;
	}
	if (keys & CHG_WEAPON) {
		struct a_type *a = type_find(player->weapon->next->type);
		player->weapon = player->weapon->next;
		player->shootpct = a->shootpct;
		keys ^= CHG_WEAPON;
	}
	if (keys & START) {
		keys ^= START;
		if(game_mode == STARTSCREEN)
			switch_game_mode(PLAYING);
	}
}

void button_press_event (GtkWidget *widget, GdkEventButton *event)
{
	if(conf->usekeyboard == TRUE)
		return;
	plshoot();
	return;
}

void handle_that_mouse()
{
	gint x, y;
	gtk_widget_get_pointer(drawing_area, &x, &y);

	// move the player to its new position
	if(x < 0) x = 0;
	if(y < 0) y = 0;
	if(x > SCREEN_WIDTH - player->type->width)
		x = SCREEN_WIDTH - player->type->width;
	if(y > SCREEN_HEIGHT - player->type->height)
		y = SCREEN_HEIGHT - player->type->height;

	player->x = x;
	player->y = y;
}

void plshoot()
{
//	if (num_of (WEAPONS ^ player->weapon->type) > 0)
//		return;

	if (player->weapon->type == BULLET) {
		if (num_of(BULLET) < conf->max_bullets)
			make_unit(player->x + player->type->width/2, player->y,
					0, 0, BULLET, 0, 0);
	} else {
		if (num_of(player->weapon->type) < player->shootpct) {
			make_unit(player->x, player->y, 0, 0,
					player->weapon->type, 0, 0);
		}
	}


}


/* code below sets up cursor */
void load_cursor(GtkWidget *widget)
{
	GdkBitmap *bitmap;
	GdkBitmap *mask;
	GdkCursor *cursor_dot;

	GdkColor black = {0, 0x0000, 0x0000, 0x0000};

	masked_bitmap_from_xpm (&bitmap, &mask, dotcursor_xpm);
	cursor_dot = gdk_cursor_new_from_pixmap (bitmap, mask, &black,
						&black, 1, 1);
	
	gdk_window_set_cursor (widget->window, cursor_dot);
}

/* this fuction was taken from Developing Linux
 * Applications with gtk+ and gdk. pretty good book...
 * AND WHY IS THIS FUNCTION IN THIS FILE ?  It's for the cursor?
 */
void masked_bitmap_from_xpm (GdkBitmap **bitmap, GdkBitmap **mask, gchar **xpm)
{
	int height, width, colors;
	char pixmap_buffer [(32 * 32)/8];
	char mask_buffer [(32 * 32)/8];
	int x, y, pix;
	int transparent_color, black_color;

	sscanf (xpm [0], "%d %d %d %d", &height, &width, &colors, &pix);

	g_assert (height == 32);
	g_assert (width  == 32);
	g_assert (colors == 3);

	transparent_color = ' ';
	black_color = '.';

	for (y = 0; y < 32; y++)
		for (x = 0; x < 32;) {
			char value = 0, maskv = 0;

			for (pix = 0; pix < 8; pix++, x++) {
				if (xpm [4+y][x] != transparent_color)
					maskv |= 1 << pix;

				if (xpm [4+y][x] != black_color)
					value |= 1 << pix;
			}
			pixmap_buffer [(y * 4 + x/8)-1] = value;
			mask_buffer [(y * 4 + x/8)-1] = maskv;
		}

	*bitmap = gdk_bitmap_create_from_data (NULL, pixmap_buffer, 32, 32);
	*mask   = gdk_bitmap_create_from_data (NULL, mask_buffer, 32, 32);
}

