/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* flarp!!! */
#include "offender.h"
#include "ui.h"
#include "package_conf.h"

#define DRAW_TIMEOUT	50

struct a_type *a_types = NULL;
GtkWidget *window = NULL;
GList *free_units = NULL;
GList *units = NULL;
struct unit *player = NULL;
GtkWidget *drawing_area = NULL;

int level = 0;

gint in_focus (GtkWidget *widget, gpointer data)
{
//	if(game_mode == PAUSED)
//		switch_game_mode(PLAYING);
	gdk_key_repeat_disable();
	return 1;
}

gint out_focus (GtkWidget *widget, gpointer data)
{
	if(game_mode != PAUSED)
		switch_game_mode(PAUSED);
	clear_keys();
	gdk_key_repeat_restore();
	return 1;
}

int main (int argc, char *argv[]) 
{
	GtkWidget *vbox;
	
	gtk_init(&argc, &argv);

	srand( (unsigned) time(NULL));

	init_config();
        
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, TRUE);
	gtk_window_set_title(GTK_WINDOW(window), PACKAGE_STRING);
	gtk_widget_set_events (window, GDK_KEY_RELEASE_MASK);

	/* vertical packing box */
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (window), vbox);
	gtk_widget_show (vbox);

	gtk_signal_connect(GTK_OBJECT (window), "destroy",
			GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
	
	create_item_factory(vbox, window);
	
	drawing_area = gtk_drawing_area_new();
	gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area),
				SCREEN_WIDTH, SCREEN_HEIGHT);

	gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK
					| GDK_LEAVE_NOTIFY_MASK
					| GDK_BUTTON_PRESS_MASK
					| GDK_POINTER_MOTION_MASK
					| GDK_POINTER_MOTION_HINT_MASK);

	/* set up signal handlers */
	gtk_signal_connect(GTK_OBJECT (drawing_area), "expose_event",
			(GtkSignalFunc) expose_event, NULL);
	gtk_signal_connect(GTK_OBJECT (drawing_area), "configure_event",
			(GtkSignalFunc) config_event, NULL);
	gtk_signal_connect(GTK_OBJECT (window), "focus_in_event",
			(GtkSignalFunc) in_focus, NULL);
	gtk_signal_connect(GTK_OBJECT (window), "focus_out_event",
			(GtkSignalFunc) out_focus, NULL);
	gtk_signal_connect(GTK_OBJECT (window), "key_press_event", 
				(GtkSignalFunc) track_keys, NULL);
	gtk_signal_connect(GTK_OBJECT (window), "key_release_event", 
				(GtkSignalFunc) release_keys, NULL);
	gtk_signal_connect(GTK_OBJECT (window), "delete_event", 
				(GtkSignalFunc) gtk_main_quit, NULL);
	gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event",
				(GtkSignalFunc) button_press_event, NULL);

	gtk_box_pack_end (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);

	gtk_widget_show (drawing_area);

	game_mode = STARTSCREEN;

	/* we need at least one thing for repaint to paint.. */
	make_known_types();

	background = make_unit_prepend (0, 0, 0, 0, BG, 1, 0);
	background->seq = STARTSCREENSEQNUM;

	player = make_unit(P1_START_X, P1_START_Y, 8, 8, PLAYER, 0, 7);

	/* the main loop */
	gtk_widget_show(window);
	gtk_timeout_add (DRAW_TIMEOUT, repaint, (gpointer) drawing_area);

	/* initialize the game */
	init_level();
	load_cursor(drawing_area);
	
	gtk_main();
	return 0;
}


void switch_game_mode(gint mode)
{
	gint orig_mode = game_mode;

	if(mode == orig_mode && mode == PAUSED)
		mode = PLAYING;
	
	game_mode = mode;

	if(mode == orig_mode)
		return;
	
	if(orig_mode == STARTSCREEN || orig_mode == GETREADY) {
		if(mode == PAUSED) {
			game_mode = orig_mode;
			return;
		}
	}

	if(orig_mode == STARTSCREEN) {
		if(mode == PLAYING) {
			new_game();
		}
	} else if(orig_mode == PAUSED) {
		if(mode == PLAYING) {
			/* This number will later be determined in levels.c on 
			 * a level by level basis. */
			background->seq = LEVELONESEQNUM;
		} else if (mode == STARTSCREEN) {
			background->seq = STARTSCREENSEQNUM;
		}
	} else if(orig_mode == PLAYING) {
		if(mode == STARTSCREEN) {
			background->seq = STARTSCREENSEQNUM;
		} else if (mode == PAUSED) {
			background->seq = PAUSED;
		}
		if(mode == GETREADY) {
			background->seq = GETREADY;
			clear_because_player_died();
			get_ready_timeout = 30;
		}
	} else if(orig_mode == GETREADY) {
		if(mode == PLAYING)
			background->seq = LEVELONESEQNUM;
	}
	
	if(game_mode != PLAYING)
		background->y = 0;

	return;
}
