/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __OFFENDER_H
#define __OFFENDER_H

#include <gtk/gtk.h>
#include <math.h>

GtkWidget *drawing_area;
#define SCREEN_WIDTH	400
#define SCREEN_HEIGHT	500

#define STRAFE_DIST	70

#define P1_START_X	200
#define P1_START_Y	430

// off the bottom of the screen
#define OFF_BOTTOM(y) (y >= SCREEN_HEIGHT)
#define ALIENS (GALIEN | BALIEN | CLOAK | BLOB | KAMI | RALIEN)

#if !HAVE_MEMCPY && HAVE_BCOPY
#define memcpy(dest, src, n) (bcopy (src, dest, n))
#endif

#define NUM_LEVELS      8
#define MAX_LIVES	10

#define STARTSCREEN 	0
#define PLAYING		1
#define PAUSED		2
#define GETREADY	3

gint game_mode;

gint current_background;

struct a_type *a_types;
GList *free_units; // list of unused alloc'ed mem
GList *units;      // all active units
struct unit *player;
struct unit *background;

gint get_ready_timeout;

#define STARTSCREENSEQNUM 	0
#define LEVELONESEQNUM		1

gint cur_level;

struct sprite {
	char **xpm_data;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
};

struct weapon {
	gint type;
	//gint available;
	struct weapon *next;
	struct weapon *prev;
};

struct a_type {
	gint type;	// types below 'unit'
	gint health;	// default health
	gint inc_x;	// more
        gint inc_y;	// defaults
	gint shootpct;	// and default shootpct for easy copying
	gint height;
	gint width;
	void (*ai) (struct unit *alien);
	gint nr_seqs;
	struct sprite *sprites;
	struct a_type *next;
	struct a_type *prev;
};

struct unit {
	gint x;
	gint y;
	gint old_x;	// to hold place while
	gint old_y;	// swerve_attacking
	gint inc_x;
	gint inc_y;
	gint health;
	gint seq;
	gint shootpct;

	struct weapon *weapon;

	// replace with a union when there are >1 formations
	gint form_swerve;
	gint form_strafe;

	struct a_type *type;
	struct unit *next;
};


  /* Unit types -- by powers of 2*/
#define GALIEN		/* POW_2(0) */ 1
#define BALIEN		/* POW_2(1) */ (1 << 1)
#define RALIEN		/* POW_2(2) */ (1 << 2)
#define BLOB		/* POW_2(3) */ (1 << 3)
#define CLOAK		/* POW_2(4) */ (1 << 4)
#define KAMI		/* POW_2(5) */ (1 << 5)
#define PLAYER		/* POW_2(6) */ (1 << 6)
#define LASER		/* POW_2(7) */ (1 << 7)
#define EXPLO		/* POW_2(8) */ (1 << 8)
#define STAR		/* POW_2(9) */ (1 << 9)
#define TANK		/* POW_2(10) */ (1 << 10)
#define NUM		/* POW_2(11) */ (1 << 11)
#define ROCKET  	/* POW_2(12) */ (1 << 12)
#define ROCKET_PUP 	/* POW_2(13) */ (1 << 13)
#define ROCKET_DSP	/* POW_2(14) */ (1 << 14)
#define HEAT_S		/* POW_2(15) */ (1 << 15)
#define HEAT_S_PUP	/* POW_2(16) */ (1 << 16)
#define HEAT_S_DSP	/* POW_2(17) */ (1 << 17)
#define GUN		/* POW_2(18) */ (1 << 18)
// skip one power of 2  /* POW_2(19) */ (1 << 19)
#define GUN_DSP		/* POW_2(20) */ (1 << 20)
#define BG		/* POW_2(21) */ (1 << 21)
#define BULLET		GUN
#define DSPS	 (ROCKET_DSP | HEAT_S_DSP | GUN_DSP)
#define WEAPONS  (ROCKET | HEAT_S | GUN)
#define POWERUPS (ROCKET_PUP | HEAT_S_PUP)

#define INC_X     0
#define INC_Y     1
#define ATK_X     2
#define TIMES     3
#define ATK_Y     4
#define TOWARDS   5
#define STRF_DIST 6
#define HEALTH    7
#define SHOOTPCT  8

struct config {
	gint diff_level;
	gint swrv_xinc;
	gint swrv_xinc_times;
	gint swrv_yinc;
	gint keyleft;
	gint keyright;
	gint keyup;
	gint keydown;
	gint keyshoot;
	gint keychg_weapon;
	gint keystart;
	gint usekeyboard;
	gint start_level;
	gint start_lives;
	gint max_bullets;
	gint galien[9];
	gint balien[9];
	gint ralien[9];
	gint blob[9];
	gint cloak[9];
	gint kami[9];
	gint tank[9];
};

struct config *conf;

enum {
	EASY,
	NORMAL,
	HARD,
};


// units.c
inline struct unit *make_unit(gint, gint, gint, gint, gint, gint, gint);
inline struct unit *make_unit_prepend(gint, gint, gint, gint, gint, gint, gint);
struct a_type *type_find(int);
void make_known_types();
struct unit *_make_unit(gint, gint, gint, gint, gint, gint, gint, gint);
inline void make_star();
inline void make_power_ups(struct unit *);
void explo_destroy(int, int);
inline void check_creature_collide(struct unit *);
inline void free_unit(struct unit *);
int change_type_sprite(gint type, struct sprite *sprite);
void rm_type_from_list(gint, gint);
gint num_of(gint);
void do_units();
struct unit *type_in_range(gint, gint, gint, gint, gint);
void change_type(gint, gint, gint, gint, gint);
void clear_because_player_died();

// levels.c
void load_level(int);
inline void next_level();
inline void new_game();
void game_over();
void init_level();

// animate.c
gint repaint (gpointer);
void sequence_sprite(GtkWidget *, struct unit *);
void draw_sprite(GtkWidget *, struct unit *);
gint expose_event (GtkWidget *, GdkEventExpose *);
gint config_event(GtkWidget *, GdkEventConfigure *);
void load_pixmap(GtkWidget *, struct a_type *);

// config.c
void config_make();
void init_config();
int *conf_a_from_type(struct config*, int);
void update_types();

// ai.c
void ai_normal(struct unit *);
void ai_blue_alien(struct unit *a);
void ai_cloak(struct unit *);
void ai_blob(struct unit *);
void ai_kami(struct unit *);

void ai_laser (struct unit *);
void ai_rocket (struct unit *);
void ai_heats (struct unit *);
void ai_dec_health (struct unit *);
gint tank_shoot (gpointer data);
void show_health (struct unit *);
void ai_dsp (struct unit *);
void ai_background (struct unit *);

// power_ups.c  powerup / weapon handling
void power_up (struct unit *, struct unit *);
struct weapon *check_weapon (struct unit *, gint);
void add_weapon (struct unit *, gint);
void remove_weapon (struct unit *, gint);
void free_all_weapons (struct unit *);

// input.c
void clear_keys();

// menu.c
void pop_up_win(char *, char *);

// offender.c
void switch_game_mode(gint);

#define make_explo(x,y) make_unit(x, y, 0, 0, EXPLO, 0, 0)

#endif
