/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/********************************************************
 * power_ups.c - providing the proper functionality
 * to render various player powers
 * ******************************************************/

#include "offender.h"
#include <stdio.h>
#include <stdlib.h>


void p_dsp_add(struct weapon *);

  /* power_up() - assign appropriate power to unit in contact */
void
power_up (u, power)
	struct unit *u, *power;
{
	gint powerup;
	struct weapon *new_wep;

	/* shift powerup type to weapon type */
	powerup = power->type->type >> 1;

	if (check_weapon (u, powerup) != NULL)
		return;				/* already have it */
	else
		add_weapon (u, powerup);	/* add new power up */

	/* find new weapon in the list */
	for (new_wep = u->weapon; !(new_wep->type & powerup);
			new_wep = new_wep->next);
	g_assert(new_wep->type & powerup);

	/* adopt new powerup shootpct */
//	u->shootpct = power->shootpct;

//	p_dsp_add(new_wep);
/*
	{
		struct weapon *tmp = u->weapon->next;
		printf("-- Received a powerup -- \n%d, %d, %d, \n",
			tmp->prev->type, tmp->type, tmp->next->type);
	}
*/
}

  /* check for weapon, return if have, NULL if not. */
struct weapon *
check_weapon (u, powerup)
	struct unit *u;
	gint powerup;
{
	struct weapon *tmp = u->weapon;
	gint curr = u->weapon->type;

	do {
		if (tmp->type == powerup)
			return tmp;
		tmp = tmp->next;
		
		if (curr == tmp->type)
			return NULL;
	} while (curr != tmp->type);
}

  /* add a weapon to the list */
void
add_weapon (u, powerup)
	struct unit *u;
	gint powerup;
{
	struct a_type *a = NULL;
	struct weapon *tmp = u->weapon;

	for (;!(tmp->next->type & (BULLET | LASER)); tmp = tmp->next);

	if (tmp->next->type & (BULLET | LASER)) {
		struct weapon *new_wep = (struct weapon *) g_malloc 
					( sizeof (struct weapon));
		if (!new_wep) {
			fprintf (stderr, "Out of Mem! exiting\n");
			exit (1);
		}

		new_wep->type = powerup;
		new_wep->next = tmp->next;
		new_wep->prev = tmp->next->prev;

		tmp->next = new_wep;
		/* take care of first adds */
		if (tmp->prev->type & (BULLET | LASER))
			tmp->prev = new_wep;
	} else {
		fprintf(stderr, "NO BULLETS!  serious problem\n");
		exit (1);
	}
}

  /* remove a weapon from the list if it's there */
void
remove_weapon (u, powerup)
	struct unit *u;
	gint powerup;
{
	struct weapon *have_wep = check_weapon (u, powerup);

	if (have_wep == NULL) {
		  /* don't have so return */
		return;
	} else {
		  /* take weapon out of list and free */
		struct weapon *w_next = have_wep->next;
		struct weapon *w_prev = have_wep->prev;
		w_prev->next = w_next;
		w_next->prev = w_prev;
		g_free (have_wep);
	}
}

  /* return unit to original state weapon-wise (ie player w/ only gun) */
void
free_all_weapons (u)
	struct unit *u;
{
	struct weapon *tmp = u->weapon;

	for(; !(tmp->type & (BULLET | LASER)); tmp = tmp->next);

	if (tmp->type & (BULLET | LASER)) {
		struct weapon *w_keep = tmp;
		struct weapon *tmp1;

		if (w_keep->next->type & (BULLET | LASER))
			return;

		for (tmp = w_keep->next, tmp1 = tmp->next;
				tmp != w_keep; tmp1 = tmp->next) {

			free (tmp);
			tmp = tmp1;
		}
		w_keep->next = w_keep;
		w_keep->prev = w_keep;

		u->weapon = w_keep;
	} else {
		fprintf (stderr, "free_all_weapons: NO BULLETS/LASERS!"
			 "serious problem\n");
		exit (1);
	}
}

/*
 * possibly unnecessary function
 * /
void
p_dsp_add(new_wep)
	struct weapon *new_wep;
{
	int n = 0;
	// shift twice from the weapon gives the weapon _DSP type *
	gint prev_wep_DSP = new_wep->type << 2;
	struct weapon *tmp = new_wep, *next;

	// set tmp to point to weapon directly after BULLET *
	for (;!(tmp->type & (BULLET | LASER)); tmp = tmp->next);
	tmp = tmp->next;

	// count weapons *
	for (next = tmp->next; !(tmp->type & (BULLET | LASER)); tmp = next) {
		next = tmp->next;
		n++;
	}

	// make display of weapons held *
	make_unit(5 + n * 22, SCREEN_HEIGHT - 24 - 5, 0, 0, prev_wep_DSP,
			1, 0);
}*/
