/*
 * Copyright (C) 2007 silas albrecht and dylan albrecht.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "offender.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <error.h>
#include <pixmaps/declare.h>

extern GtkWidget *window;

#define ON_SCREEN(x,y) (x <= SCREEN_WIDTH && x >= 0 && \
			y <= SCREEN_HEIGHT && y >= 0 )
#define IS_PLAYER(x) (x->type->type & PLAYER)
#define IS_ALIEN(x) (x->type->type & ALIENS)
#define IS_POWERUP(x) (x->type->type & POWERUPS)
#define IS_WEAPON(x) (x->type->type & WEAPONS)
#define IS_DSP(x) (x->type->type & DSPS)

struct unit *get_free_unit();
struct a_type *type_new(gint, struct sprite *, gint, gint, gint, gint,
                        void (*ai) (struct unit *));

// bit ugly ;)
struct unit *fresh_unit(gint x, gint y, gint inc_x, gint inc_y,
			gint utype, gint health, gint shootpct)
{
	gint type = utype;
	struct sprite *p_sprite = NULL;
	struct unit *unit = get_free_unit();

	if(!type) return NULL;

	unit->x = x;
	unit->y = y;
	unit->old_x = x;
	unit->old_y = y;
	unit->seq = 0;
	unit->form_swerve = 0;
	unit->form_strafe = 0;
	unit->type = type_find(type);
	unit->weapon = (struct weapon *) malloc (sizeof (struct weapon));

	/* set proper weapon depending on type of unit */
	if (IS_ALIEN (unit)) {
		unit->weapon->type = LASER;
		unit->weapon->next = unit->weapon;
		unit->weapon->prev = unit->weapon;
	} else if (IS_PLAYER (unit)) {
		unit->weapon->type = BULLET;
		unit->weapon->next = unit->weapon;
		unit->weapon->prev = unit->weapon;
	} else if (IS_WEAPON (unit)) {
		unit->weapon->type = unit->type->type;
		unit->weapon->next = NULL;
		unit->weapon->prev = NULL;
	} else if (IS_POWERUP (unit)) {
		unit->weapon->type = unit->type->type >> 1;
		unit->weapon->next = NULL;
		unit->weapon->prev = NULL;
	} else if (IS_DSP (unit)) {
		unit->weapon->type = unit->type->type >> 2;
		unit->weapon->next = NULL;
		unit->weapon->prev = NULL;
	} else {
		free (unit->weapon);
		unit->weapon = NULL;
	}

	if(!unit->type)
		return NULL;

	unit->health = health ?health :unit->type->health;
	unit->shootpct = shootpct ?shootpct :unit->type->shootpct;
	unit->inc_x = inc_x ?inc_x :unit->type->inc_x;
	unit->inc_y = inc_y ?inc_y :unit->type->inc_y;

	return unit;
}

inline struct unit *make_unit_prepend(gint x, gint y, gint inc_x, gint inc_y,
		gint type, gint health, gint shootpct)
{
	struct unit *n = fresh_unit(x,y,inc_x,inc_y,type,health,shootpct);
	units = g_list_prepend(units, n);
	return n;
}


inline struct unit *make_unit(gint x, gint y, gint inc_x, gint inc_y,
		gint type, gint health, gint shootpct)
{
	struct unit *n = fresh_unit(x,y,inc_x,inc_y,type,health,shootpct);
	units = g_list_append(units, n);
	return n;
}

inline void make_star()
{
	if ((rand() % 100) < 15)
		make_unit((rand() % 400), 0, 0, ((rand() % 2)+1)*3-1, STAR, 0, 0);
}

inline void make_power_ups(struct unit *a)
{
	if ((rand() % 20) == 0){
		make_unit(a->x, a->y, 0, 4, ROCKET_PUP, 0, 0);
	}
	if ((rand() % 20) == 0) {
		make_unit(a->x, a->y, 0, 4, HEAT_S_PUP, 0, 0);
	}
}

// raaghh!! :)
void explo_destroy(int x, int y)
{
	int n = 5;
	make_explo(x+n, y+n);
	make_explo(x-n, y+n);
	make_explo(x+n, y-n);
	make_explo(x-n, y-n);
	make_explo(x, y);
}

/* if an alien hit the player or the player hit an alien...
   hurt both.  also handle powerups
 */
void check_creature_collide(struct unit *a)
{
	gint check_type;
	struct unit *hit;

	if(IS_PLAYER(a))
		check_type = ALIENS | TANK | POWERUPS;
	else if(IS_ALIEN(a))
		check_type = PLAYER;
	else
		return;

	hit = type_in_range(check_type, a->x, a->y, a->x + a->type->width,
						a->y + a->type->height);
	if (hit) {
		  /* NEVER remove a TANK!  :) */
		if(!(hit->type->type & TANK))
			hit->health--;
		  /* Power UP! */
		if (hit->type->type & POWERUPS) {
			power_up (a, hit);	/* power 'hit' to unit 'a' */
			return;
		}

		  /* awww  player alien collision */
		if(IS_PLAYER(a) || IS_ALIEN(a)) {
			a->health--;
			if (hit->health == 0)
				switch_game_mode(STARTSCREEN);
			else
				switch_game_mode(GETREADY);


			if(a->health > 0)
				make_explo(a->x, a->y);
			else	// dead ship :)
				explo_destroy(a->x, a->y);
		}
	}
}

// This function is very fragile ... *please* be careful..
void do_units()
{
	GList *node = units, *next;
	struct unit *a;

	for (; node; node = next) {
		next = node->next;
		a = (struct unit *) node->data;

		if (a->health <= 0) {
			if (IS_ALIEN (a))
				make_power_ups(a);
			free_unit (a);
			continue;
		}

		if (a->type->ai)
			a->type->ai(a);

		if(game_mode != PLAYING)
			return;

		g_assert(a != NULL);

		if(!IS_PLAYER(a)) {
                        a->x += a->inc_x;
                        a->y += a->inc_y;
                }

		check_creature_collide(a); // check if anybody's been hit

		if (a->health <= 0 || player->health <= 0) {
			if (IS_PLAYER (a) || player->health <= 0) {
				game_over();
				return;
			}

			free_unit (a);
			continue;
		}

		if(a->type->type == BG)
			draw_sprite(drawing_area, a);
		else if ( !IS_ALIEN(a) && !ON_SCREEN(a->x, a->y))
			free_unit(a);
		else
			sequence_sprite(drawing_area, a);
	}

	make_star();
	if(!num_of(ALIENS))
		next_level();
}

struct unit *type_in_range(gint type, gint x1, gint y1, gint x2, gint y2)
{
	GList *node = units, *next;
	struct unit *item = NULL;

	for (; node; node = next) {
		next = node->next;
		item = (struct unit *) node->data;
		if(!(type & item->type->type))
			continue;

		if(( (x1 >= item->x && x2 <= item->x + item->type->width) ||
		     (x1 <= item->x && x2 >= item->x + item->type->width) ||
		     (x2 >= item->x && x1 <= item->x + item->type->width) ||
		     (x2 <= item->x && x1 >= item->x + item->type->width) ) &&
		   ( (y1 >= item->y && y2 <= item->y + item->type->height) ||
		     (y1 <= item->y && y2 >= item->y + item->type->height) ||
		     (y2 >= item->y && y1 <= item->y + item->type->height) ||
		     (y2 <= item->y && y1 >= item->y + item->type->height) ))
			return item;
	}

	return NULL;
}

void make_known_types()
{
	g_assert(a_types == NULL);
	// type, sprite[], shootpct, health, inc_x, inc_y, func
	type_new(GALIEN, sprite_galien, 0, 1, -4, 0, 	ai_normal);
	type_new(BALIEN, sprite_balien, 0, 1, -4, 0, 	ai_blue_alien);
	type_new(RALIEN, sprite_ralien, 0, 1, -4, 0, 	ai_normal);
	type_new(CLOAK,  sprite_cloak,  0, 1, 1, 1,     ai_cloak);	
	type_new(BLOB,   sprite_blob,   0, 4, 0, 0,     ai_blob);
	type_new(KAMI,   sprite_kami,   0, 3, -4, 0,    ai_kami);
	type_new(PLAYER, sprite_hero,   0, 3, 0, 0,     NULL);
	type_new(LASER,  sprite_laser,	0, 1, 0, 10,	ai_laser);
	type_new(BULLET, sprite_bullet, 0, 1, 0, -10,   ai_laser);
	type_new(EXPLO,  sprite_explo,	0, 5, 0, 0, 	ai_dec_health);
	type_new(STAR,   sprite_star,	0, 1, 0, 0,	NULL);
	type_new(TANK,	 sprite_tank,   0, 1, 0, 0,	NULL);
	type_new(NUM,	 sprite_nums,   0, 1, 0, 0,	ai_dsp);
	type_new(ROCKET_PUP, sprite_rocket_pup, 3, 1, 0, 4, NULL);
	type_new(ROCKET, sprite_rocket_fire, 3, 1, 0, -2, ai_rocket);
	type_new(ROCKET_DSP, sprite_display_r, 3, 1, 0, 0, ai_dsp);
	type_new(HEAT_S_PUP, sprite_heats_pup, 8, 1, 0, 4, NULL);
	type_new(HEAT_S, sprite_heats_fire, 8, 1, 0, -4, ai_heats);
	type_new(HEAT_S_DSP, sprite_display_h, 8, 1, 0, 0, ai_dsp);
	type_new(GUN_DSP, sprite_display_g, 7, 1, 0, 0, ai_dsp);
	type_new(BG, sprite_backgrounds, 0, 1, 0, 4, ai_background);

}

void clear_because_player_died()
{
	GList *node = units, *next;
	struct unit *a;

	free_all_weapons(player);
	
	for (; node; node = next) {
		next = node->next;
		a = (struct unit *) node->data;
		
		if(IS_ALIEN(a)) {
			a->x = a->old_x;
			a->y = a->old_y;
		} else if(a->type->type & (BULLET |  LASER |
			POWERUPS | WEAPONS)) {
			free_unit(a);
		} else if(IS_PLAYER(a)) {
			a->x = P1_START_X;
			a->y = P1_START_Y; 
		}
	}

	return;
}

/* list management till eof :) */
/* find free mem to use or else make it */
struct unit *get_free_unit()
{
	struct unit *ret;

	/* all units are used up (or something very strange is going on :) */
	if (free_units == NULL) {
		ret = (struct unit *)g_malloc(sizeof(struct unit));
		if(ret == NULL) {
			g_print("ERR: couldn't alloc mem for unit\n");
			exit(1);
		}
	} else {
		ret = (struct unit *)free_units->data;
		free_units = g_list_remove(free_units, ret);
	}

	return ret;
}

inline void free_unit(struct unit *goner)
{
	units = g_list_remove(units, goner);
	memset((struct units *) goner, 0, sizeof(struct unit));
	free_units = g_list_append(free_units, goner);
}

gint num_of(gint type)
{
	GList *node = units;
	struct unit *item;
	int num = 0;

	while (node) {
		item = (struct unit *) node->data;
		node = node->next;
		if (type & item->type->type)
			num++;
	}
	return num;
}

void rm_type_from_list(gint type, gint all)
{
	GList *node = units;
	struct unit *item;

	while (node) {
		item = (struct unit *) node->data;
		node = node->next;

		if (all || (type & item->type->type))
			free_unit(item);
	}
}

int type_reg (struct a_type *node)
{
	if(a_types)
		a_types->prev = node;
	node->next = a_types;
	a_types = node;

	return 1;
}

struct a_type *type_find(int type)
{
	struct a_type *tmp = a_types;

	for (; tmp; tmp = tmp->next)
		if (tmp->type == type)
			break;

	return tmp;
}

// will be used for power-ups, player health display, etc,.. :)
int change_type_sprite(gint type, struct sprite *sprite)
{
	struct a_type *a = type_find(type);
	if (a == NULL)	// erm.. no luck :)
		return 0;
	a->sprites = sprite;
	return 1;
}

struct a_type *type_new(gint type, struct sprite *sprite, gint shootpct,
			gint health, gint inc_x, gint inc_y,
			void (*ai) (struct unit *alien))
{
	struct a_type *new = (struct a_type *)g_malloc (sizeof(struct a_type));
	new->type = type;
	
	new->health = health;
	new->inc_x = inc_x;
	new->inc_y = inc_y;
	new->shootpct = shootpct;
	new->ai = ai;
	new->sprites = sprite;
	load_pixmap(window, new);

	type_reg(new);

	return new;
}

void change_type (gint type, gint inc_x, gint inc_y, gint health, 
		gint shootpct)
{
	struct a_type *to_change = type_find (type);

	to_change->inc_x = inc_x;
	to_change->inc_y = inc_y;
	to_change->health = health;
	to_change->shootpct = shootpct;

	return;
}


